==> cargo-sonar-0.7.0.crate: cargo-sonar-0.7.0/src/deny.rs <==
use crate::{cargo::Lockfile, sonar};
use eyre::{Context as _, Result};
use skip_error::SkipError;
use std::{
    fs::File,
    io::{BufRead as _, BufReader},
    path::PathBuf,
};
use tracing::{error, info};

const DENY_ENGINE: &str = "deny";

#[derive(Debug, serde::Deserialize)]
pub struct GraphNode {
    name: String,
    version: String,
    #[serde(default)]
    kind: String,
    #[serde(default)]
    repeat: bool,
    #[serde(default)]
    parents: Vec<GraphNode>,
}

#[derive(Debug, serde::Deserialize)]
pub struct Label {
    line: usize,
    column: usize,
    message: String,
    span: String,
}

#[derive(Debug, serde::Deserialize)]
#[serde(untagged)]
enum DenyReport {
    Advisory {
        advisory: rustsec::advisory::Metadata,
    },
    License {
        code: String,
        graphs: Vec<GraphNode>,
        message: String,
        labels: Vec<Label>,
    },
}

#[derive(Debug)]
pub struct Deny<'ranges> {
    json: PathBuf,
    lockfile: &'ranges Lockfile,
}

impl<'ranges> Deny<'ranges> {
    pub fn new<P>(json: P, lockfile: &'ranges Lockfile) -> Self
    where
        P: Into<PathBuf>,
    {
        Self {
            json: json.into(),
            lockfile,
        }
    }
    fn to_issue(&self, deny_report: &DenyReport) -> Option<sonar::Issue> {
        match deny_report {
            DenyReport::Advisory { advisory } => {
                let rule_id = advisory.id.to_string();
                let message = format!(
                    "{} (see https://github.com/rustsec/advisory-db/blob/main/crates/{}/{}.md)",
                    advisory.title, advisory.package, advisory.id
                );
                let file_path = self.lockfile.lockfile_path.to_string_lossy().to_string();
                let dependency = self.lockfile.dependencies.get(advisory.package.as_str());
                let text_range = dependency
                    .map(|ranges| &ranges.range)
                    .map(sonar::TextRange::clone)
                    .unwrap_or_else(|| {
                        error!("failed to find a corresponding text range for the advisory related to '{}'. This is probably a bug in `cargo-sonar`, if you can please report an issue at https://gitlab.com/woshilapin/cargo-sonar/-/issues, we'd be happy to try and fix it.", advisory.package);
                        sonar::TextRange {
                            start_line: 1,
                            end_line: 1,
                            start_column: 0,
                            end_column: 1,
                        }
                    });
                let primary_location = sonar::Location {
                    message,
                    file_path,
                    text_range,
                };
                let secondary_locations = Vec::new();
                let issue = sonar::Issue {
                    engine_id: DENY_ENGINE.to_string(),
                    rule_id,
                    severity: sonar::Severity::Major,
                    r#type: sonar::IssueType::Vulnerability,
                    primary_location,
                    secondary_locations,
                };
                Some(issue)
            }
            DenyReport::License {
                code,
                graphs,
                message,
                labels,
            } => {
                let graph_node = graphs.get(0).unwrap();
                let dependency = self.lockfile.dependencies.get(graph_node.name.as_str());
                let file_path = self.lockfile.lockfile_path.to_string_lossy().to_string();
                let text_range = dependency
                    .map(|ranges| &ranges.range)
                    .map(sonar::TextRange::clone)
                    .unwrap_or_else(|| {
                        error!("failed to find a corresponding text range for this license warning related to '{}'. This is probably a bug in `cargo-sonar`, if you can please report an issue at https://gitlab.com/woshilapin/cargo-sonar/-/issues, we'd be happy to try and fix it.", graph_node.name);
                        sonar::TextRange {
                            start_line: 1,
                            end_line: 1,
                            start_column: 0,
                            end_column: 1,
                        }
                    });
                let message = labels.iter().fold(message.to_owned(), |message, label| {
                    format!("{}.\n`{}` {}", message, label.span, label.message)
                });
                let primary_location = sonar::Location {
                    message,
                    file_path,
                    text_range,
                };
                let secondary_locations = Vec::new();
                let issue = sonar::Issue {
                    engine_id: DENY_ENGINE.to_string(),
                    rule_id: code.to_owned(),
                    severity: sonar::Severity::Info,
                    r#type: sonar::IssueType::CodeSmell,
                    primary_location,
                    secondary_locations,
                };
                Some(issue)
            }
        }
    }
}

impl std::convert::TryInto<sonar::Report> for Deny<'_> {
    type Error = eyre::Error;

    fn try_into(self) -> Result<sonar::Report> {
        let file = File::open(&self.json).with_context(|| {
            format!(
                "failed to open 'cargo-deny' report from '{:?}' file",
                self.json
            )
        })?;
        let reader = BufReader::new(file);
        let issues: sonar::Report = reader
            .lines()
            .flatten()
            .flat_map(|line| serde_json::from_str::<serde_json::Value>(&line))
            .flat_map(|value| value.get("fields").map(ToString::to_string))
            .map(|s| {
                serde_json::from_str::<DenyReport>(&s)
                    .with_context(|| format!("failed to deserialize '{}'", s))
            })
            .skip_error_and_debug()
            .flat_map(|diagnostic| self.to_issue(&diagnostic))
            .collect();
        info!("{} sonar issues created", issues.len());
        Ok(issues)
    }
}

==> cargo-sonar-0.8.1.crate: cargo-sonar-0.8.1/src/deny.rs <==
use crate::{cargo::Lockfile, sonar};
use eyre::{Context as _, Result};
use skip_error::SkipError;
use std::{
    fs::File,
    io::{BufRead as _, BufReader},
    path::PathBuf,
};
use tracing::{error, info};

const DENY_ENGINE: &str = "deny";

#[derive(Debug, serde::Deserialize)]
pub struct GraphNode {
    name: String,
    version: String,
    #[serde(default)]
    kind: String,
    #[serde(default)]
    repeat: bool,
    #[serde(default)]
    parents: Vec<GraphNode>,
}

#[derive(Debug, serde::Deserialize)]
pub struct Label {
    line: usize,
    column: usize,
    message: String,
    span: String,
}

#[derive(Debug, serde::Deserialize)]
#[serde(untagged)]
enum DenyReport {
    Advisory {
        advisory: rustsec::advisory::Metadata,
    },
    License {
        code: String,
        graphs: Vec<GraphNode>,
        message: String,
        labels: Vec<Label>,
    },
}

#[derive(Debug)]
pub struct Deny<'lock> {
    json: PathBuf,
    lockfile: &'lock Lockfile,
}

impl<'lock> Deny<'lock> {
    pub fn new<P>(json: P, lockfile: &'lock Lockfile) -> Self
    where
        P: Into<PathBuf>,
    {
        Self {
            json: json.into(),
            lockfile,
        }
    }
    fn to_issue(&self, deny_report: &DenyReport) -> Option<sonar::Issue> {
        match deny_report {
            DenyReport::Advisory { advisory } => {
                let rule_id = format!("{}::{}", DENY_ENGINE, advisory.id);
                let message = format!(
                    "{} (see https://github.com/rustsec/advisory-db/blob/main/crates/{}/{}.md)",
                    advisory.title, advisory.package, advisory.id
                );
                let file_path = self.lockfile.lockfile_path.to_string_lossy().to_string();
                let dependency = self.lockfile.dependencies.get(advisory.package.as_str());
                let text_range = dependency
                    .map(|ranges| &ranges.range)
                    .map(sonar::TextRange::clone)
                    .unwrap_or_else(|| {
                        error!("failed to find a corresponding text range for the advisory related to '{}'. This is probably a bug in `cargo-sonar`, if you can please report an issue at https://gitlab.com/woshilapin/cargo-sonar/-/issues, we'd be happy to try and fix it.", advisory.package);
                        sonar::TextRange {
                            start_line: 1,
                            end_line: 1,
                            start_column: 0,
                            end_column: 1,
                        }
                    });
                let primary_location = sonar::Location {
                    message,
                    file_path,
                    text_range,
                };
                let secondary_locations = Vec::new();
                let issue = sonar::Issue {
                    engine_id: DENY_ENGINE.to_string(),
                    rule_id,
                    severity: sonar::Severity::Major,
                    r#type: sonar::IssueType::Vulnerability,
                    primary_location,
                    secondary_locations,
                };
                Some(issue)
            }
            DenyReport::License {
                code,
                graphs,
                message,
                labels,
            } => {
                let graph_node = graphs.get(0).unwrap();
                let dependency = self.lockfile.dependencies.get(graph_node.name.as_str());
                let file_path = self.lockfile.lockfile_path.to_string_lossy().to_string();
                let text_range = dependency
                    .map(|ranges| &ranges.range)
                    .map(sonar::TextRange::clone)
                    .unwrap_or_else(|| {
                        error!("failed to find a corresponding text range for this license warning related to '{}'. This is probably a bug in `cargo-sonar`, if you can please report an issue at https://gitlab.com/woshilapin/cargo-sonar/-/issues, we'd be happy to try and fix it.", graph_node.name);
                        sonar::TextRange {
                            start_line: 1,
                            end_line: 1,
                            start_column: 0,
                            end_column: 1,
                        }
                    });
                let message = labels.iter().fold(message.to_owned(), |message, label| {
                    format!("{}.\n`{}` {}", message, label.span, label.message)
                });
                let primary_location = sonar::Location {
                    message,
                    file_path,
                    text_range,
                };
                let secondary_locations = Vec::new();
                let issue = sonar::Issue {
                    engine_id: DENY_ENGINE.to_string(),
                    rule_id: format!("{}::{}", DENY_ENGINE, code),
                    severity: sonar::Severity::Info,
                    r#type: sonar::IssueType::CodeSmell,
                    primary_location,
                    secondary_locations,
                };
                Some(issue)
            }
        }
    }
}

impl std::convert::TryInto<sonar::Report> for Deny<'_> {
    type Error = eyre::Error;

    fn try_into(self) -> Result<sonar::Report> {
        let file = File::open(&self.json).with_context(|| {
            format!(
                "failed to open 'cargo-deny' report from '{:?}' file",
                self.json
            )
        })?;
        let reader = BufReader::new(file);
        let issues: sonar::Report = reader
            .lines()
            .flatten()
            .flat_map(|line| serde_json::from_str::<serde_json::Value>(&line))
            .flat_map(|value| value.get("fields").map(ToString::to_string))
            .map(|s| {
                serde_json::from_str::<DenyReport>(&s)
                    .with_context(|| format!("failed to deserialize '{}'", s))
            })
            .skip_error_and_debug()
            .flat_map(|diagnostic| self.to_issue(&diagnostic))
            .collect();
        info!("{} sonar issues created", issues.len());
        Ok(issues)
    }
}
